# a simple Rich Tcl Text viewer
#

source ttd.tcl

proc main {} {
    global argv argc
    global widgets
    global filename

    set filename ""

    Init .

    if {$argc == 0} return

    if {$argc == 1} {
	command-open $widgets(text) [lindex $argv 0]
    }
}

proc Init {{w .}} {
    global widgets

    if {$w == "."} {
	set parent ""
    } else {
	set parent $w
    }

    wm title $w "ttdView"

    set widgets(menubar) [menu $parent.menubar]
    set widgets(text) $parent.text
    set widgets(vsb)  $parent.vsb
    set widgets(hsb)  $parent.hsb

    $w configure -menu $widgets(menubar)

    set widgets(fileMenu) [menu $parent.menubar.fileMenu -tearoff false]
    set widgets(editMenu) [menu $parent.menubar.editMenu -tearoff false]

    $widgets(fileMenu) add command -label "Open..." \
	    -underline 0 -command [list command-open $widgets(text)]
    $widgets(fileMenu) add command -label "Save..." \
	    -underline 0 -command [list command-save $widgets(text)]
    $widgets(fileMenu) add separator
    $widgets(fileMenu) add command -label "Exit" \
	    -underline 2 -command [list command-exit $widgets(text)]
    $widgets(editMenu) add command -label "Cut"  \
	    -underline 2 -command [list command-cut $widgets(text)]
    $widgets(editMenu) add command -label "Copy" \
	    -underline 0 -command [list command-copy $widgets(text)]
    $widgets(editMenu) add command -label "Paste" \
	    -underline 0 -command [list command-paste $widgets(text)]

    $widgets(menubar) add cascade -label "File" \
	    -underline 0 \
	    -menu $widgets(fileMenu)
    $widgets(menubar) add cascade -label "Edit" \
	    -underline 0 \
	    -menu $widgets(editMenu)
    
    text $widgets(text) \
	    -width 80 \
	    -height 40 \
	    -wrap word \
	    -yscrollcommand [list $parent.vsb set] \
	    -xscrollcommand [list $parent.hsb set] 

    scrollbar $widgets(vsb) \
	    -orient vertical \
	    -command [list $parent.text yview]

    scrollbar $widgets(hsb) \
	    -orient horizontal \
	    -command [list $parent.text xview]

    grid $widgets(text) -row 1 -column 0 -sticky nsew
#    grid $widgets(hsb) -row 2 -column 0  -sticky ew
    grid $widgets(vsb) -row 1 -column 1  -sticky ns

    grid rowconfigure    $w 0 -weight 0
    grid rowconfigure    $w 1 -weight 1
    grid rowconfigure    $w 2 -weight 0

    grid columnconfigure $w 0 -weight 1
    grid columnconfigure $w 1 -weight 0
}

proc command-save {w} {
    global filename
    global widgets

    set file [tk_getSaveFile \
	    -defaultextension {.ttd}  \
	    -initialdir . \
	    -filetypes {{{Rich Tcl Text} *.ttd TEXT} {all *.* TEXT}} \
	    -initialfile $filename \
	    -title "Save..."]

    if {$file != ""} {
	set filename $file
	set id [open $filename w]
	set data [::ttd::get $widgets(text)]
	puts -nonewline $id $data
	close $id
	wm title . "$file - Tcl/Tk Text Dump Viewer"
    }
}

proc command-add-tag {w} {
    global tags

    if {![info exists .addtag]} {
	toplevel .addtag
	wm group .addtag .
	wm transient .addtag true

	listbox .addtag.lb
	scrollbar .addtag.vsb -command {.addtab.lb yview}
	grid .addtag.lb -row 0 column 0
	grid .addtag.vsb
    }
}
    
proc command-open {w {file ""}} {
    global filename
    global widgets

    if {$file == ""} {
	set file [tk_getOpenFile \
		-defaultextension {.ttd}  \
		-initialdir . \
		-filetypes {{{Rich Tcl Text} *.ttd TEXT} {all *.* TEXT} } \
		-initialfile $filename \
		-title "Open..."]
    }

    if {$file == ""} {return}

    $w delete 1.0 end
 
    set filename $file
    wm title . "$filename - Tcl/Tk Text Dump Viewer"
    set id [open $filename]
    set ttd [read $id]
    close $id
    if {[catch {ttd::insert $w $ttd} message]} {
	tk_messageBox -icon info \
		-message $message \
		-title "The file cannot be opened" \
		-type ok \
		-parent $w
    }
    $w mark set insert 1.0
}

proc command-cut {w} {
    event generate $w <<Cut>>
}

proc command-copy {w} {
    event generate $w <<Copy>>
}

proc command-paste {w} {
    event generate $w <<Paste>>
}

proc command-exit {} {
    exit
}

main

